#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import logging


def port_in_range(port_number: int) -> bool:
    u"""Checks if the port is within the valid range
        from 0 to 65536."""

    if isinstance(port_number, int):
        if port_number >= 0 and port_number < 65536:
            logging.debug('Port within range')
            return True
        else:
            logging.error('Port not within valid ' +
                          'range from 0 to 65536')
            return False
    else:
        raise ValueError('Port has to be an integer.')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# python standard library:
import logging
from typing import Union
from urllib.parse import urlparse


def is_url(url: str,
           require_specific_schemes: Union[tuple, None] = None) -> bool:
    u"""Very basic check if the URL fulfills basic conditions
        ("LGTM"). Will not try to connect."""
    parsed = urlparse(url)

    if parsed.scheme == '':
        logging.error('The URL has no scheme (like http or https)')
        return False
    if require_specific_schemes:
        if parsed.scheme not in require_specific_schemes:
            logging.error('Scheme %s not supported.', parsed.scheme)
            return False

    if parsed.netloc == '':
        logging.error('URL is missing or malformed.')
        return False

    return True



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import datetime
import logging
import re


def date_exists(year: int,
                month: int,
                day: int) -> bool:
    u"""Check if a date given by three integers is valid
        i.e exists in the calendar. """
    try:
        # int() will convert something like '01' to 1
        year = int(year)
        month = int(month)
        day = int(day)
    except ValueError:
        logging.error('Could not convert date parts to integer.')
        return False

    try:
        datetime.datetime(year, month, day)
    except ValueError:
        logging.error('Provided date does not exist in the calendar.')
        return False
    return True


def date_en_long_to_iso(date_string: str) -> str:
    u""" Changes long format English date to short form date. """
    date_string = date_string.strip()
    regex_long_date_en = re.compile("(?P<monthL>[a-zA-Z\.]{3,9})\s+(?P<day>\d{1,2})(th)?,\s*(?P<year>\d\d\d\d)")
    try:
        match = re.search(regex_long_date_en, date_string)
        if match:
            matchYear = match.group('year')
            matchMonth = match.group('monthL')
            matchDay = match.group('day')
        else:
            raise AttributeError('No date provided')
    except AttributeError:
        logging.error('Malformed date')
        raise

    # add a zero to day if <10
    if(len(matchDay) == 1):
        matchDay = '0' + matchDay
    months = {
        'January': '01',
        'Jan.': '01',
        'February': '02',
        'Feb.': '02',
        'March': '03',
        'Mar.': '03',
        'April': '04',
        'Apr.': '04',
        'May': '05',
        'June': '06',
        'Jun.': '06',
        'July': '07',
        'Jul.': '07',
        'August': '08',
        'Aug.': '08',
        'September': '09',
        'Sep.': '09',
        'October': '10',
        'Oct.': '10',
        'November': '11',
        'Nov.': '11',
        'December': '12',
        'Dec.': '12'
        }
    try:
        matchMonth = months[str(matchMonth).lower().capitalize()]
    except KeyError:
        # String for month matched the regular expression but is no
        # recognized month.
        logging.error('Do not recognize month.')
        raise

    if date_exists(int(matchYear), int(matchMonth), int(matchDay)):
        return(f"{matchYear}-{matchMonth}-{matchDay}")
    else:
        raise ValueError('Provided date is invalid.')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import logging
import re


def is_aws_s3_bucket_name(bucket_name: str) -> bool:
    u"""returns True if bucket name is well-formed for AWS S3 buckets

    Applying the rules set here:
    https://docs.aws.amazon.com/AmazonS3/latest/dev/BucketRestrictions.html
    """

    # Lengthy code which could be written as a single regular expression.
    # However written in this way to provide useful error messages.
    if len(bucket_name) < 3:
        logging.error('Any AWS bucket name has to be at least 3 ' +
                      'characters long.')
        return False
    if len(bucket_name) > 63:
        logging.error('The provided bucket name for AWS exceeds the ' +
                      'maximum length of 63 characters.')
        return False
    if not re.match(r"^[a-z0-9\-\.]*$", bucket_name):
        logging.error('The AWS bucket name contains invalid characters.')
        return False
    if re.match(r"[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}",
                bucket_name):
        # Check if the bucket name resembles an IPv4 address.
        # No need to check IPv6 as the colon is not an allowed character.
        logging.error('An AWS must not resemble an IP address.')
        return False
    if re.match(r"([a-z0-9][a-z0-9\-]*[a-z0-9]\.)*[a-z0-9][a-z0-9\-]*[a-z0-9]",
                bucket_name):
        # Must start with a lowercase letter or number
        # Bucket names must be a series of one or more labels.
        # Adjacent labels are separated by a single period (.).
        # Each label must start and end with a lowercase letter or a number.
        # => Adopted the answer provided by Zak (zero or more labels
        # followed by a dot) found here:
        # https://stackoverflow.com/questions/50480924
        return True

    logging.error('Invalid AWS bucket name.')
    return False



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from userprovided import cloud
from userprovided import date
from userprovided import hash
from userprovided import mail
from userprovided import port
from userprovided import url


name = "userprovided"
__version__ = "0.5.4"
__author__ = "Rüdiger Voigt"



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="userprovided",
    version="0.5.4",
    author="Rüdiger Voigt",
    author_email="projects@ruediger-voigt.eu",
    description="A library to check user input.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/RuedigerVoigt/userprovided",
    packages=setuptools.find_packages(),
    python_requires=">=3.6",
    install_requires=[],
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Development Status :: 4 - Beta",
        "Intended Audience :: Science/Research",
        "Intended Audience :: Developers",
        "Topic :: Security",
        "Topic :: Utilities",
        "Topic :: Software Development :: Quality Assurance"
    ],
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import hashlib
import logging


def hash_available(hash_method: str,
                   fail_on_deprecated: bool = True) -> bool:
    u"""Checks if the supplied hashing algorithm is available.
        Will raise an exception if not available or deprecated."""

    if hash_method == '' or hash_method is None:
        raise ValueError('No hash method provided')

    # Is the chosen method available and supported?
    hash_method = hash_method.strip()

    if fail_on_deprecated:
        if hash_method in ('md5', 'sha1'):
            raise ValueError('The supplied hash method %s is deprecated!')

    if hash_method in hashlib.algorithms_available:
        logging.debug('Hash method %s is available.', hash_method)
        return True
    else:
        return False



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# python standard library:
import logging
import re


def is_email(mailaddress: str) -> bool:
    u"""Very basic check if the email address has a valid format."""

    if mailaddress is None or mailaddress == '':
        logging.warning('No mail address supplied.')
        return False
    else:
        mailaddress = mailaddress.strip()
        if not re.match(r"^[^\s@]+@[^\s@]+\.[a-zA-Z]+", mailaddress):
            logging.error('The supplied mailaddress %s has an unknown ' +
                          'format.', mailaddress)
            return False
        else:
            logging.debug('%s seems to have a valid format', mailaddress)
            return True
